/*
 * Copyright (C)  NHR@FAU, University Erlangen-Nuremberg.
 * All rights reserved.
 * Use of this source code is governed by a MIT-style
 * license that can be found in the LICENSE file.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "discretization.h"
#include "parameter.h"
#include "progress.h"
#include "solver.h"
#include "timing.h"

int main(int argc, char** argv)
{
    double timeStart, timeStop;
    Parameter p;
    Discretization d;
    Solver s;
    
    initParameter(&p);

    if (argc != 2) {
        printf("Usage: %s <configFile>\n", argv[0]);
        exit(EXIT_SUCCESS);
    }

    readParameter(&p, argv[1]);
    printParameter(&p);
    initDiscretization(&d, &p);
    initSolver(&s, &d, &p);
    
#ifndef VERBOSE
    initProgress(d.te);
#endif

    double tau = d.tau;
    double te  = d.te;
    double t   = 0.0;
    int nt     = 0;

    timeStart = getTimeStamp();
    while (t <= te) {
        if (tau > 0.0) computeTimestep(&d);
        setBoundaryConditions(&d);
        setSpecialBoundaryCondition(&d);
        computeFG(&d);
        computeRHS(&d);
        if (nt % 100 == 0) normalizePressure(&d);
        solve(&s, d.p, d.rhs);
        adaptUV(&d);
        t += d.dt;
        nt++;

#ifdef VERBOSE
        printf("TIME %f , TIMESTEP %f\n", t, solver.dt);
#else
        printProgress(t);
#endif
    }
    timeStop = getTimeStamp();
    stopProgress();
    printf("Solution took %.2fs\n", timeStop - timeStart);
    writeResult(&d);
    return EXIT_SUCCESS;
}
